/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : inews.c
 *  Author    : I.Lea
 *  Created   : 17-03-92
 *  Updated   : 28-03-92
 *  Notes     : NNTP builtin version of inews
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *              You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"
#include	"nntp.h"


int submit_inews (name)
	char *name;
{	
	int	ret_code = FALSE;

#ifdef NNTP_INEWS
	char	*ptr;
	char	full_name[128];
	char	host_name[128];
	char	line[NNTP_STRLEN];
	FILE	*fp;
	int	respcode;

	if ((fp = fopen (name, "r")) == NULL) {
		return (ret_code);
	}

	/*
	 * Send POST command to NNTP server
	 */
	put_server ("post");

	/*
	 * Receive CONT_POST or ERROR response code from NNTP server
	 */
	if ((respcode = get_respcode ()) != CONT_POST) {
		error_message ("%s", nntp_respcode (respcode));
		debug_nntp ("submit_inews", nntp_respcode (respcode));
		return (ret_code);
	}

	/*
	 * Find users name and hosts name
	 */
	if ((ptr = (char *) getenv ("NAME")) != (char *) 0) {
		strcpy (full_name, ptr);
	} else {	
		strcpy (full_name, myentry->pw_gecos);
		if ((ptr = (char *) strchr (full_name, ','))) {
			*ptr = '\0';			
		}
	}

#if defined(BSD) || defined(sinix) || defined(RS6000)
	gethostname (host_name, sizeof (host_name)); 
#else	
	uname (host_name);
#endif
	
	if (debug == 2) {
		sprintf (msg, "NAME=[%s] HOST=[%s]", full_name, host_name);
		wait_message (msg);
		sleep (2);
	}	
	 
	/*
	 * Send Path: and From: article headers
	 */
	sprintf (line, "Path: %s!%s", host_name, userid);
	put_server (line);
	sprintf (line, "From: %s@%s (%s)", userid, host_name, full_name);
	put_server (line);

	/*
	 * Send article 1 line at a time ending with "."
	 */
	while (fgets (line, sizeof (line), fp) != NULL) {
		if (line[strlen (line)-1] == '\n') {
			line[strlen (line)-1] = '\0';
		}
		put_server (line);	
	}
	
	put_server (".");

	/*
	 * Receive OK_POSTED or ERROR response code from NNTP server
	 */
	if ((respcode = get_respcode ()) != OK_POSTED) {
		error_message ("%s", nntp_respcode (respcode));
		debug_nntp ("submit_inews", nntp_respcode (respcode));
		return (ret_code);
  	}
  	
	ret_code = TRUE;

#endif /* NNTP_ABLE */

	return (ret_code);
}

